//----------------------------------------------------------------------------
//----------------------------------------------------------------------------

#ifndef PARTICLE_H
#define PARTICLE_H

#include <stdio.h>
#include <stdlib.h>

#include "Sprite.hh"
#include "Wall.hh"

#include <math.h>
#include <string.h>
#include <sys/time.h>

#include <GL/glut.h>

#include <vector>

using namespace std;

//----------------------------------------------------------------------------
//----------------------------------------------------------------------------

#define DEFAULT_MASS         0.01
#define DEFAULT_RADIUS       0.1
#define DEFAULT_RESTITUTION  0.9

class Particle2D
{
public:

  Point2D p;                        // current position (m)
  Point2D v;                        // current velocity (m / s)
  Point2D a;                        // current acceleration (m / s / s)
  double m;                         // mass (kg)
  double r;                         // radius (m)
  double restitution;               // 0-1

  Point2D old_p;                    // position before this timestep
  Point2D old_v;                    // velocity before this timestep

  Point2D inter;                    // location of last wall intersection
  bool collides_with_walls;         // only check for wall collisions if this is true
  bool hit_wall_flag;               // has this particle ever hit a wall?

  Sprite *sprite;                   // live appearance
  float red, green, blue;           // default "base color" of sprite

  Particle2D(double, double, double, double, double, double, double, bool, int, float = 1.0, float = 1.0, float = 1.0);

  void draw();
  void update(double);
  void set(double, double, double, double);
  void add_acceleration(double, double);
  bool detect_wall_collision(Wall2D *, double &);
  void handle_wall_collision(Wall2D *, double);
  void compute_wall_collisions();

};

//----------------------------------------------------------------------------
//----------------------------------------------------------------------------

#endif
